<?php

/* --------------------------------------------------------------
   InfoPageContent.inc.php 2019-04-15
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class InfoPageContent
 *
 * This class represents an info page content
 *
 * @category   System
 * @package    Content
 */
class InfoPageContent
    implements ContentAttributesInterface, ContentHeadingAttributeInterface, ContentTextAttributeInterface, ContentDownloadFileAttributeInterface, ContentMetaAttributesInterface, ContentNameAttributeInterface
{
    /**
     * Content type
     *
     * @var string
     */
    protected $type;
    
    /**
     * Content position
     *
     * @var string
     */
    protected $position;
    
    /**
     * Content status
     *
     * @var bool
     */
    protected $status;
    
    /**
     * Localized content titles
     *
     * @var ContentTitleCollection
     */
    protected $titles;
    
    /**
     * Localized content headings
     *
     * @var ContentHeadingCollection
     */
    protected $headings;
    
    /**
     * Localized content texts
     *
     * @var ContentHeadingCollection
     */
    protected $texts;
    
    /**
     * Localized content download files
     *
     * @var ContentHeadingCollection
     */
    protected $downloads;
    
    /**
     * Localized content meta titles
     *
     * @var ContentMetaTitleCollection
     */
    protected $metaTitles;
    
    /**
     * Localized content meta keywords
     *
     * @var ContentMetaKeywordsCollection
     */
    protected $metaKeywords;
    
    /**
     * Localized content OpenGraph images
     *
     * @var ContentOpengraphImageCollection
     */
    protected $opengraphImages;
    
    /**
     * Localized content meta descriptions
     *
     * @var ContentMetaDescriptionCollection
     */
    protected $metaDescriptions;
    
    /**
     * "Allow Robots" status
     *
     * @var bool
     */
    protected $allowRobots;
    
    /**
     * Content site map information
     *
     * @var ContentSitemap
     */
    protected $sitemap;
    
    /**
     * Localized content URL rewrites
     *
     * @var ContentUrlRewriteCollection
     */
    protected $urlRewrites;
    
    /**
     * Localized content URL keywords
     *
     * @var ContentUrlKeywordsCollection
     */
    protected $urlKeywords;
    
    /**
     * Localized content names
     *
     * @var ContentNameCollection
     */
    protected $names;
    
    
    /**
     * InfoPageContent constructor
     *
     * @param ContentPositionInterface $pagePosition Page position
     * @param ContentStatusInterface $status Activation status
     * @param ContentTitleCollection $titles Localized titles
     * @param ContentHeadingCollection $headings Localized headings
     * @param ContentTextCollection $texts Localized texts
     * @param ContentDownloadFileCollection $downloads Localized download files
     * @param ContentMetaTitleCollection $metaTitles Localized meta titles
     * @param ContentMetaKeywordsCollection $metaKeywords Localized meta keywords
     * @param ContentOpengraphImageCollection $opengraphImages Localized OpenGraph images
     * @param ContentMetaDescriptionCollection $metaDescriptions Localized meta descriptions
     * @param ContentAllowRobotsStatus $allowRobotsStatus "Allow robots" status
     * @param ContentSitemap $sitemap Sitemap information
     * @param ContentUrlRewriteCollection $urlRewrites Localized URL rewrites
     * @param ContentUrlKeywordsCollection $urlKeywords Localized URL keywords
     * @param ContentNameCollection $names Localized names
     */
    public function __construct(
        ContentPositionInterface $pagePosition,
        ContentStatusInterface $status,
        ContentTitleCollection $titles,
        ContentHeadingCollection $headings,
        ContentTextCollection $texts,
        ContentDownloadFileCollection $downloads,
        ContentMetaTitleCollection $metaTitles,
        ContentMetaKeywordsCollection $metaKeywords,
        ContentOpengraphImageCollection $opengraphImages,
        ContentMetaDescriptionCollection $metaDescriptions,
        ContentAllowRobotsStatus $allowRobotsStatus,
        ContentSitemap $sitemap,
        ContentUrlRewriteCollection $urlRewrites,
        ContentUrlKeywordsCollection $urlKeywords,
        ContentNameCollection $names
    ) {
        $this->type = ContentType::createForContent()->type();
        $this->position = $pagePosition->position();
        $this->status = $status->isActive();
        $this->titles = $titles;
        $this->headings = $headings;
        $this->texts = $texts;
        $this->downloads = $downloads;
        $this->metaTitles = $metaTitles;
        $this->metaKeywords = $metaKeywords;
        $this->opengraphImages = $opengraphImages;
        $this->metaDescriptions = $metaDescriptions;
        $this->allowRobots = $allowRobotsStatus->isAllowed();
        $this->sitemap = $sitemap;
        $this->urlRewrites = $urlRewrites;
        $this->urlKeywords = $urlKeywords;
        $this->names = $names;
    }
    
    
    /**
     * Return the content type
     *
     * @return string
     */
    public function type()
    {
        return $this->type;
    }
    
    
    /**
     * Return the content position
     *
     * @return string
     */
    public function position()
    {
        return $this->position;
    }
    
    
    /**
     * Return whether the content is active
     *
     * @return bool
     */
    public function isActive()
    {
        return $this->status;
    }
    
    
    /**
     * Return the localized content titles
     *
     * @return ContentTitleCollection
     */
    public function titles()
    {
        return $this->titles;
    }
    
    
    /**
     * Return the localized content headings
     *
     * @return ContentHeadingCollection
     */
    public function headings()
    {
        return $this->headings;
    }
    
    
    /**
     * Return the localized content texts
     *
     * @return ContentTextCollection
     */
    public function texts()
    {
        return $this->texts;
    }
    
    
    /**
     * Return the localized content download files
     *
     * @return ContentDownloadFileCollection
     */
    public function downloads()
    {
        return $this->downloads;
    }
    
    
    /**
     * Return the localized content meta titles
     *
     * @return ContentMetaTitleCollection
     */
    public function metaTitles()
    {
        return $this->metaTitles;
    }
    
    
    /**
     * Return the localized content meta titles
     *
     * @return ContentMetaKeywordsCollection
     */
    public function metaKeywords()
    {
        return $this->metaKeywords;
    }
    
    
    /**
     * Return the localized content meta descriptions
     *
     * @return ContentMetaDescriptionCollection
     */
    public function metaDescriptions()
    {
        return $this->metaDescriptions;
    }
    
    
    /**
     * Return the localized content URL keywords
     *
     * @return ContentUrlKeywordsCollection
     */
    public function urlKeywords()
    {
        return $this->urlKeywords;
    }
    
    
    /**
     * Return the localized content URL rewrites
     *
     * @return ContentUrlRewriteCollection
     */
    public function urlRewrites()
    {
        return $this->urlRewrites;
    }
    
    
    /**
     * Return the content site map information
     *
     * @return ContentSitemap
     */
    public function sitemap()
    {
        return $this->sitemap;
    }
    
    
    /**
     * Return whether robots are allowed
     *
     * @return bool
     */
    public function isAllowingRobots()
    {
        return $this->allowRobots;
    }
    
    
    /**
     * Return the localized OpenGraph images
     *
     * @return ContentOpengraphImageCollection
     */
    public function opengraphImages()
    {
        return $this->opengraphImages;
    }
    
    
    /**
     * Return the localized content names
     *
     * @return ContentNameCollection
     */
    public function names()
    {
        return $this->names;
    }
}